package com.sun.org.apache.xml.internal.dtm.ref;

import com.sun.org.apache.xml.internal.utils.PrefixResolver;
import com.sun.org.apache.xml.internal.dtm.DTMIterator;
import com.sun.org.apache.xml.internal.dtm.DTMFilter;
import org.w3c.dom.Document;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import jdk.xml.internal.JdkXmlUtils;
import com.sun.org.apache.xml.internal.utils.SuballocatedIntVector;
import org.xml.sax.SAXException;
import com.sun.org.apache.xml.internal.dtm.ref.dom2dtm.DOM2DTMdefaultNamespaceDeclarationNode;
import org.w3c.dom.Attr;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import com.sun.org.apache.xml.internal.utils.XMLStringFactory;
import com.sun.org.apache.xml.internal.dtm.ref.dom2dtm.DOM2DTM;
import com.sun.org.apache.xml.internal.utils.WrappedRuntimeException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.SAXNotRecognizedException;
import com.sun.org.apache.xml.internal.dtm.ref.sax2dtm.SAX2RTFDTM;
import com.sun.org.apache.xml.internal.dtm.ref.sax2dtm.SAX2DTM;
import com.sun.org.apache.xml.internal.utils.SystemIDResolver;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.dom.DOMSource;
import com.sun.org.apache.xml.internal.dtm.DTMWSFilter;
import javax.xml.transform.Source;
import com.sun.org.apache.xml.internal.dtm.DTMException;
import com.sun.org.apache.xml.internal.res.XMLMessages;
import org.xml.sax.helpers.DefaultHandler;
import com.sun.org.apache.xml.internal.utils.XMLReaderManager;
import com.sun.org.apache.xml.internal.dtm.DTM;
import com.sun.org.apache.xml.internal.dtm.DTMManager;

public class DTMManagerDefault extends DTMManager
{
    private static final boolean DUMPTREE = false;
    private static final boolean DEBUG = false;
    protected DTM[] m_dtms;
    int[] m_dtm_offsets;
    protected XMLReaderManager m_readerManager;
    protected DefaultHandler m_defaultHandler;
    private ExpandedNameTable m_expandedNameTable;
    
    public synchronized void addDTM(DTM dtm, int id) {
        this.addDTM(dtm, id, 0);
    }
    
    public synchronized void addDTM(DTM dtm, int id, int offset) {
        if (id >= 65536) {
            throw new DTMException(XMLMessages.createXMLMessage("ER_NO_DTMIDS_AVAIL", (Object[])null));
        }
        int oldlen = this.m_dtms.length;
        if (oldlen <= id) {
            int newlen = Math.min(id + 256, 65536);
            DTM[] new_m_dtms = new DTM[newlen];
            System.arraycopy(this.m_dtms, 0, new_m_dtms, 0, oldlen);
            this.m_dtms = new_m_dtms;
            int[] new_m_dtm_offsets = new int[newlen];
            System.arraycopy(this.m_dtm_offsets, 0, new_m_dtm_offsets, 0, oldlen);
            this.m_dtm_offsets = new_m_dtm_offsets;
        }
        this.m_dtms[id] = dtm;
        this.m_dtm_offsets[id] = offset;
        dtm.documentRegistration();
        return;
    }
    
    public synchronized int getFirstFreeDTMID() {
        int n = this.m_dtms.length;
        int i = 1;
        while (i < n) {
            if (null == this.m_dtms[i]) {
                return i;
            }
            ++i;
        }
        return n;
    }
    
    public DTMManagerDefault() {
        this.m_dtms = new DTM[256];
        this.m_dtm_offsets = new int[256];
        this.m_readerManager = null;
        this.m_defaultHandler = new DefaultHandler();
        this.m_expandedNameTable = new ExpandedNameTable();
    }
    
    @Override
    public synchronized DTM getDTM(Source source, boolean unique, DTMWSFilter whiteSpaceFilter, boolean incremental, boolean doIndexing) {
        XMLStringFactory xstringFactory = this.m_xsf;
        int dtmPos = this.getFirstFreeDTMID();
        int documentID = dtmPos << 16;
        if (null != source && source instanceof DOMSource) {
            DOM2DTM dtm2 = new DOM2DTM(this, (DOMSource)source, documentID, whiteSpaceFilter, xstringFactory, doIndexing);
            this.addDTM(dtm2, dtmPos, 0);
            return dtm2;
        }
        boolean isSAXSource = null == source || source instanceof SAXSource;
        boolean isStreamSource = null != source && source instanceof StreamSource;
        if (isSAXSource || isStreamSource) {
            XMLReader reader = null;
            try {
                InputSource xmlSource;
                if (null == source) {
                    xmlSource = null;
                }
                else {
                    reader = this.getXMLReader(source);
                    xmlSource = SAXSource.sourceToInputSource(source);
                    String urlOfSource = xmlSource.getSystemId();
                    if (null != urlOfSource) {
                        try {
                            urlOfSource = SystemIDResolver.getAbsoluteURI(urlOfSource);
                        }
                        catch (Exception e) {
                            System.err.println("Can not absolutize URL: " + urlOfSource);
                        }
                        xmlSource.setSystemId(urlOfSource);
                    }
                }
                SAX2DTM dtm;
                if (source == null && unique && !incremental && !doIndexing) {
                    dtm = new SAX2RTFDTM(this, source, documentID, whiteSpaceFilter, xstringFactory, doIndexing);
                }
                else {
                    dtm = new SAX2DTM(this, source, documentID, whiteSpaceFilter, xstringFactory, doIndexing);
                }
                this.addDTM(dtm, dtmPos, 0);
                boolean haveXercesParser = null != reader && reader.getClass().getName().equals("com.sun.org.apache.xerces.internal.parsers.SAXParser");
                if (haveXercesParser) {
                    incremental = true;
                }
                if (this.m_incremental && incremental) {
                    IncrementalSAXSource coParser = null;
                    if (haveXercesParser) {
                        try {
                            coParser = (IncrementalSAXSource)Class.forName("com.sun.org.apache.xml.internal.dtm.ref.IncrementalSAXSource_Xerces").newInstance();
                        }
                        catch (Exception ex) {
                            ex.printStackTrace();
                            coParser = null;
                        }
                    }
                    if (coParser == null) {
                        if (null == reader) {
                            coParser = new IncrementalSAXSource_Filter();
                        }
                        else {
                            IncrementalSAXSource_Filter filter = new IncrementalSAXSource_Filter();
                            filter.setXMLReader(reader);
                            coParser = filter;
                        }
                    }
                    dtm.setIncrementalSAXSource(coParser);
                    if (null == xmlSource) {
                        return dtm;
                    }
                    if (null == reader.getErrorHandler()) {
                        reader.setErrorHandler(dtm);
                    }
                    reader.setDTDHandler(dtm);
                    try {
                        coParser.startParse(xmlSource);
                    }
                    catch (RuntimeException re2) {
                        dtm.clearCoRoutine();
                        throw re2;
                    }
                    catch (Exception e2) {
                        dtm.clearCoRoutine();
                        throw new WrappedRuntimeException(e2);
                    }
                }
                else {
                    if (null == reader) {
                        return dtm;
                    }
                    reader.setContentHandler(dtm);
                    reader.setDTDHandler(dtm);
                    if (null == reader.getErrorHandler()) {
                        reader.setErrorHandler(dtm);
                    }
                    try {
                        reader.setProperty("http://xml.org/sax/properties/lexical-handler", dtm);
                    }
                    catch (SAXNotRecognizedException ex2) {}
                    catch (SAXNotSupportedException ex3) {}
                    try {
                        reader.parse(xmlSource);
                    }
                    catch (RuntimeException re) {
                        dtm.clearCoRoutine();
                        throw re;
                    }
                    catch (Exception e) {
                        dtm.clearCoRoutine();
                        throw new WrappedRuntimeException(e);
                    }
                }
                return dtm;
            }
            finally {
                if (reader != null && (!this.m_incremental || !incremental)) {
                    reader.setContentHandler(this.m_defaultHandler);
                    reader.setDTDHandler(this.m_defaultHandler);
                    reader.setErrorHandler(this.m_defaultHandler);
                    try {
                        reader.setProperty("http://xml.org/sax/properties/lexical-handler", null);
                    }
                    catch (Exception ex4) {}
                }
                this.releaseXMLReader(reader);
            }
        }
        throw new DTMException(XMLMessages.createXMLMessage("ER_NOT_SUPPORTED", new Object[] { source }));
    }
    
    @Override
    public synchronized int getDTMHandleFromNode(Node node) {
        if (null == node) {
            throw new IllegalArgumentException(XMLMessages.createXMLMessage("ER_NODE_NON_NULL", (Object[])null));
        }
        if ((node instanceof DTMNodeProxy)) {
            return ((DTMNodeProxy)node).getDTMNodeNumber();
        }
        for (int max = this.m_dtms.length, i = 0; i < max; ++i) {
            DTM thisDTM = this.m_dtms[i];
            if (null != thisDTM && thisDTM instanceof DOM2DTM) {
                int handle = ((DOM2DTM)thisDTM).getHandleOfNode(node);
                if (handle != -1) {
                    return handle;
                }
            }
        }
        Node root = node;
        for (Node p = (root.getNodeType() == 2) ? ((Attr)root).getOwnerElement() : root.getParentNode(); p != null; p = p.getParentNode()) {
            root = p;
        }
        DOM2DTM dtm = (DOM2DTM)this.getDTM(new DOMSource(root), false, null, true, true);
        int handle2;
        if ((node instanceof DOM2DTMdefaultNamespaceDeclarationNode)) {
            handle2 = dtm.getHandleOfNode(((Attr)node).getOwnerElement());
            handle2 = dtm.getAttributeNode(handle2, node.getNamespaceURI(), node.getLocalName());
        }
        else {
            handle2 = dtm.getHandleOfNode(node);
        }
        if (-1 == handle2) {
            throw new RuntimeException(XMLMessages.createXMLMessage("ER_COULD_NOT_RESOLVE_NODE", (Object[])null));
        }
        return handle2;
    }
    
    public synchronized XMLReader getXMLReader(Source inputSource) {
        try {
            XMLReader reader = (inputSource instanceof SAXSource) ? ((SAXSource)inputSource).getXMLReader() : null;
            if (null == reader) {
                if (this.m_readerManager == null) {
                    this.m_readerManager = XMLReaderManager.getInstance(super.overrideDefaultParser());
                }
                reader = this.m_readerManager.getXMLReader();
                return reader;
            }
            else {
                return reader;
            }
        }
        catch (SAXException se) {
            throw new DTMException(se.getMessage(), se);
        }
    }
    
    public synchronized void releaseXMLReader(XMLReader reader) {
        if (this.m_readerManager != null) {
            this.m_readerManager.releaseXMLReader(reader);
        }
    }
    
    @Override
    public synchronized DTM getDTM(int nodeHandle) {
        try {
            return this.m_dtms[nodeHandle >>> 16];
        }
        catch (ArrayIndexOutOfBoundsException e) {
            if (nodeHandle == -1) {
                return null;
            }
            throw e;
        }
    }
    
    @Override
    public synchronized int getDTMIdentity(DTM dtm) {
        if ((dtm instanceof DTMDefaultBase)) {
            DTMDefaultBase dtmdb = (DTMDefaultBase)dtm;
            if (dtmdb.getManager() == this) {
                return dtmdb.getDTMIDs().elementAt(0);
            }
            return -1;
        }
        else {
            int n = this.m_dtms.length;
            int i = 0;
            while (i < n) {
                DTM tdtm = this.m_dtms[i];
                if (tdtm == dtm && this.m_dtm_offsets[i] == 0) {
                    return i << 16;
                }
                ++i;
            }
            return -1;
        }
    }
    
    @Override
    public synchronized boolean release(DTM dtm, boolean shouldHardDelete) {
        if ((dtm instanceof SAX2DTM)) {
            ((SAX2DTM)dtm).clearCoRoutine();
        }
        if ((dtm instanceof DTMDefaultBase)) {
            SuballocatedIntVector ids = ((DTMDefaultBase)dtm).getDTMIDs();
            for (int j = ids.size() - 1; j >= 0; --j) {
                this.m_dtms[ids.elementAt(j) >>> 16] = null;
            }
        }
        else {
            int i = this.getDTMIdentity(dtm);
            if (i >= 0) {
                this.m_dtms[i >>> 16] = null;
            }
        }
        dtm.documentRelease();
        return true;
    }
    
    @Override
    public synchronized DTM createDocumentFragment() {
        try {
            DocumentBuilderFactory dbf = JdkXmlUtils.getDOMFactory(super.overrideDefaultParser());
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.newDocument();
            Node df = doc.createDocumentFragment();
            return this.getDTM(new DOMSource(df), true, null, false, false);
        }
        catch (Exception e) {
            throw new DTMException(e);
        }
    }
    
    @Override
    public synchronized DTMIterator createDTMIterator(int whatToShow, DTMFilter filter, boolean entityReferenceExpansion) {
        return null;
    }
    
    @Override
    public synchronized DTMIterator createDTMIterator(String xpathString, PrefixResolver presolver) {
        return null;
    }
    
    @Override
    public synchronized DTMIterator createDTMIterator(int node) {
        return null;
    }
    
    @Override
    public synchronized DTMIterator createDTMIterator(Object xpathCompiler, int pos) {
        return null;
    }
    
    public ExpandedNameTable getExpandedNameTable(DTM dtm) {
        return this.m_expandedNameTable;
    }
}
